import { __decorate } from 'tslib';
import { html } from 'lit';
import { property, queryAssignedElements } from 'lit/decorators.js';
import SgdsElement from '../../base/sgds-element.js';
import { classMap } from 'lit/directives/class-map.js';
import css_248z from './sidenav.js';

/**
 * @summary The side navigation is used to display a list of links to move between pages within a related category.
 * It is used as a secondary form of navigation where the primary navigation is located hierachically above the page frame.
 * Maximum three levels of navigations are allowed.
 *
 * @slot default - Default slot for SgdsSidenavItem element.
 * @cssproperty --sidenav-sticky-top - set the top value of the sticky sidenav. Defaults to 0rem
 */
class SgdsSidenav extends SgdsElement {
    constructor() {
        super(...arguments);
        /** Apply position sticky to the sidenav */
        this.sticky = false;
    }
    /** @internal */
    get items() {
        return [...(this.defaultNodes || [])].filter((node) => typeof node.tagName !== "undefined");
    }
    async onToggle(event) {
        const target = event.target;
        const isSidenavLink = target.tagName === "SGDS-SIDENAV-LINK";
        // Let the event pass through the DOM so that it can be
        // prevented from the outside if a user so desires.
        if (event.defaultPrevented || isSidenavLink) {
            // No toggling when the user prevents it.
            return;
        }
        const items = [...this.items];
        if (items && !items.length) {
            // no toggling when there aren't items.
            return;
        }
    }
    render() {
        return html `
      <nav class=${classMap({ sticky: this.sticky })}>
        <div>
          <slot @click=${this.onToggle}></slot>
        </div>
      </nav>
    `;
    }
}
SgdsSidenav.styles = [...SgdsElement.styles, css_248z];
__decorate([
    property({ type: Boolean, attribute: true })
], SgdsSidenav.prototype, "sticky", void 0);
__decorate([
    queryAssignedElements()
], SgdsSidenav.prototype, "defaultNodes", void 0);

export { SgdsSidenav, SgdsSidenav as default };
//# sourceMappingURL=sgds-sidenav.js.map
